"""Test the Django AMP Renderer package."""

# Third Party
from mock import MagicMock, Mock, patch

# Django AMP Renderer
# AMP Renderer
from django_amp_renderer.middleware import AMPRenderingMiddleware

settings = Mock()
settings.AMP_RUNTIME_VERSION = "X0123456789"
settings.AMP_RUNTIME_STYLES = "body{background:pink;}"


BOILERPLATE_HEADER_KEY = "Boilerplate-Status"
NON_TRIGGERING = "<!doctype html><html><head></head><body></body></html>".encode("utf-8")
triggering_1 = """
    <!doctype html>
    <html>
        <head>
            <script async src="https://cdn.ampproject.org/v0.js" ></script>
        </head>
        <body>
        </body>
    </html>
"""
triggering_2 = """
    <!doctype html>
    <html>
        <head>
            <script async src="https://cdn.ampproject.org/v0.js" ></script>
        </head>
        <body>
        </body>
    </html>
"""
triggering_3 = """
    <!doctype html>
    <html>
        <head>
            <script async src="https://cdn.ampproject.org/v0.js" ></script>
        </head>
        <body>
        </body>
    </html>
"""
triggering_4 = """
    <!doctype html>
    <html>
        <head>
            <script
                type='module'
                crossorigin="anonymous"
                async src="https://cdn.ampproject.org/lts/v0.js"
            ></script>
        </head>
        <body>
        </body>
    </html>
"""

TRIGGERING = tuple(
    [html.encode("utf-8") for html in (triggering_1, triggering_2, triggering_3, triggering_4)],
)


@patch("django_amp_renderer.middleware.settings", new=settings)
class TestMiddleware:
    """Test the Django AMP Rendering middleware."""

    def setup_method(self, method):
        """Set up a middleware with mock request values."""
        request = Mock()
        request.META = MagicMock()
        request.GET = {}
        self.request = request

        response = MagicMock()
        response.has_header = MagicMock(return_value=True)
        headers = {"Content-Type": "text/html"}
        response.__getitem__.side_effect = headers.__getitem__
        self.response = response

        self.middleware = AMPRenderingMiddleware(self._get_response)

    def teardown_method(self, method):
        """Clean up."""
        del self.request
        del self.response
        del self.middleware

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_non_triggering(self, mock_amp_renderer):
        """Test a response that doesn't trigger the renderer."""
        self.response.content = NON_TRIGGERING

        self.middleware.process_response(self.request, self.response)
        assert not mock_amp_renderer.called
        self.response.__setitem__.assert_not_called()

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_triggering_1(self, mock_amp_renderer):
        """Test #1."""
        self._setup_renderer(mock_amp_renderer)

        self.response.content = TRIGGERING[0]

        self.middleware.process_response(self.request, self.response)

        assert mock_amp_renderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(BOILERPLATE_HEADER_KEY, "Removed")

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_triggering_2(self, mock_amp_renderer):
        """Test #2."""
        self._setup_renderer(mock_amp_renderer)

        self.response.content = TRIGGERING[1]

        self.middleware.process_response(self.request, self.response)

        assert mock_amp_renderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(BOILERPLATE_HEADER_KEY, "Removed")

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_triggering_3(self, mock_amp_renderer):
        """Test #3."""
        self._setup_renderer(mock_amp_renderer)

        self.response.content = TRIGGERING[2]

        self.middleware.process_response(self.request, self.response)

        assert mock_amp_renderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(BOILERPLATE_HEADER_KEY, "Removed")

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_triggering_4(self, mock_amp_renderer):
        """Test #4."""
        self._setup_renderer(mock_amp_renderer)

        self.response.content = TRIGGERING[3]

        self.middleware.process_response(self.request, self.response)

        assert mock_amp_renderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(BOILERPLATE_HEADER_KEY, "Removed")

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_not_html(self, mock_amp_renderer):
        """Test a non-HTML response."""
        self.response.content = TRIGGERING[0]

        headers = {"Content-Type": "application/json"}
        self.response.__getitem__.side_effect = headers.__getitem__

        self.middleware.process_response(self.request, self.response)

        assert not mock_amp_renderer.called
        self.response.__setitem__.assert_not_called()

    @patch("django_amp_renderer.middleware.AMPRenderer")
    def test_non_unicode(self, mock_amp_renderer):
        """Test a non-unicode response."""
        self.response.content = "õ".encode("cp857")

        self.middleware.process_response(self.request, self.response)

        assert not mock_amp_renderer.called
        self.response.__setitem__.assert_not_called()

    def _setup_renderer(self, mock_amp_renderer):
        self.renderer = MagicMock()
        self.renderer.render = MagicMock(return_value="Yay! HTML!")
        mock_amp_renderer.return_value = self.renderer

    def _get_response(self):
        pass



"""Define middleware for applying AMPRenderer to the output of Django views."""

# Standard Library
import re

# Third Party
from amp_renderer import AMPRenderer

# Django
from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from django.utils.encoding import DjangoUnicodeDecodeError


class AMPRenderingMiddleware(MiddlewareMixin):
    """Apply AMPRenderer to the output of Django views."""

    # These will be passed along to AMP Renderer
    should_strip_comments = False
    should_trim_attrs = False

    def process_response(self, request, response):
        """Process the response after the view has rendered it."""
        if not response.has_header("Content-Type") or "text/html" not in response["Content-Type"]:
            return response

        try:
            response_content = response.content.decode("utf-8").strip()
        except (DjangoUnicodeDecodeError, UnicodeDecodeError):
            return response

        # If the script is included in the document, then apply the
        # transformations that the script would eventually happen on the
        # client.

        # Caveats:
        #   - This only applies the middleware if AMP v0 is included directly
        #     from the https://cdn.ampproject.org/v0.(m?)js.
        #   - If using RTVs or some other method, this won’t apply as written.
        #   - This regex doesn’t check that quotes are balanced, or fully
        #     confirm that the import script is valid.

        attribute_patterns = [
            "defer",
            "async",
            "type=['\"]?module['\"]?",
            "crossorigin(=['\"]?anonymous['\"]?)?",
        ]

        regex = (
            "<script({opt})*\\s+src=['\"]https://cdn"  # noqa: WPS342
            + "\\.ampproject\\.org/(lts/)?v0\\.m?js['\"]"  # noqa: WPS342
            + r"({opt})*\s*>\s*</script>"
        ).format(
            opt="|".join(r"\s+{}".format(pattern) for pattern in attribute_patterns),
        )

        if not re.search(regex, response_content):
            return response

        boilerplate_header = "Ignored"

        parser = AMPRenderer(
            runtime_version=settings.AMP_RUNTIME_VERSION,
            runtime_styles=settings.AMP_RUNTIME_STYLES,
        )

        parser.should_strip_comments = self.should_strip_comments
        parser.should_trim_attrs = self.should_trim_attrs
        response_content = parser.render(response_content)

        response.content = response_content
        response["Content-Length"] = len(response.content)

        if parser.no_boilerplate:
            boilerplate_header = "Removed"

        # Set a header on the response for downstream code to know
        # whether the boilerplate was removed.
        response["Boilerplate-Status"] = boilerplate_header

        return response



"""Define metadata for Django AMP Renderer."""

# Third Party
import setuptools

with open("README.md", "r") as readme_file:
    long_description = readme_file.read()

setuptools.setup(
    name="django-amp-renderer",
    version="2.0.1",
    author="Chase Finch",
    author_email="chase@finch.email",
    description="Middleware for applying AMP Renderer to the output of a request in Django.",
    keywords=["Django", "AMP", "AMP Optimizer", "server-side rendering"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chasefinch/django-amp-renderer",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "amp-renderer>=2.0.0",
        "django>=2.2.0",
    ],
)
